@@ -33,7 +33,7 @@ class TupleFormatReader(object):
             if isinstance(axis, basestring):
                 ax = filter(lambda x: x._name == axis, aslist(self.root.Axes.Axis))[0]
             else:
-                ax = self.root.Axes.Axis[axis]
+                ax = aslist(self.root.Axes.Axis)[axis]
             res = []
             for tup in aslist(getattr(ax.Tuples, "Tuple", [])):
                 res.append(tup.Member)
@@ -141,13 +141,13 @@ class TupleFormatReader(object):
                 axisranges[0][1].append(cell)
             else:
                 if isinstance(properties, basestring):
-                    d = cell.get(properties, 
-                                       "Property %s does not exist" % property)
+                    d = getattr(cell, properties, 
+                                       None)
                 else:
                     d = {}
                     for prop in aslist(properties): 
-                        d[prop] = cell.get(prop, 
-                                           "Property %s does not exist" % property)
+                        d[prop] = getattr(cell, prop, 
+                                           None)
                 axisranges[0][1].append(d)
             
             # advance to next requested element in slice
