@@ -5,8 +5,7 @@ Created on 22.05.2012
 '''
 from requests.auth import AuthBase
 from requests.compat import urlparse
-from requests import get as reqget
-from requests import session
+from requests import __version__ as reqver
 import re
 import logging
 
@@ -53,11 +52,14 @@ class HTTPKerberosAuth(AuthBase):
     rx = re.compile('(?:.*,)*\s*Negotiate\s*([^,]*),?', re.I)
     auth_header = 'www-authenticate'
 
-    def __init__(self, as_user=None, spn=None, gssflags=k.GSS_C_MUTUAL_FLAG|k.GSS_C_SEQUENCE_FLAG):
-        self.retried = 0
-        self.context = None
-        self.gssflags=gssflags
-        self.spn = spn
+    def __init__(self, as_user=None, spn=None, gssflags=k.GSS_C_MUTUAL_FLAG|k.GSS_C_SEQUENCE_FLAG
+                 , sslverify=True, proxies=None):
+        self.sslverify = sslverify
+        self.proxies   = proxies
+        self.retried   = 0
+        self.context   = None
+        self.gssflags  = gssflags
+        self.spn       = spn
         if as_user:
             self.gss_step = s4u2p.authGSSImpersonationStep
             self.gss_response = s4u2p.authGSSImpersonationResponse
@@ -100,6 +102,8 @@ class HTTPKerberosAuth(AuthBase):
     def handle_401(self, r):
         """Takes the given response and tries kerberos negotiation, if needed."""
 
+        log.info("kerberosauth: handle_401...")
+
         ret = r
         r.request.deregister_hook('response', self.handle_401)
 
@@ -107,6 +111,8 @@ class HTTPKerberosAuth(AuthBase):
         firstround = False
         if neg_value is not None:
             
+            log.info("kerberosauth: neg_value not none")
+            
             if self.context is None:
                 spn = self.get_spn(r)
                 result, self.context = self.gss_init(spn, self.gssflags)
@@ -116,7 +122,7 @@ class HTTPKerberosAuth(AuthBase):
                     return None
 
                 firstround = False
-                log.debug("gss_init() succeeded")
+                log.info("gss_init() succeeded")
 
             result = self.gss_step(self.context, neg_value)
 
@@ -126,19 +132,41 @@ class HTTPKerberosAuth(AuthBase):
                 log.warning("gss_step returned result %d" % result)
                 return None
 
-            log.debug("gss_step() succeeded")
+            log.info("gss_step() succeeded")
 
-            if result == k.AUTH_GSS_CONTINUE or (result == k.AUTH_GSS_COMPLETE and not (self.gssflags & k.GSS_C_MUTUAL_FLAG) and firstround):
+            if result == k.AUTH_GSS_CONTINUE or \
+                    (result == k.AUTH_GSS_COMPLETE and \
+                         not (self.gssflags & k.GSS_C_MUTUAL_FLAG) and firstround):
                 response = self.gss_response(self.context)
-                r.request.headers['Authorization'] = "Negotiate %s" % response
-                r.request.send(anyway=True)
-                _r = r.request.response
-                _r.history.append(r)
+                req=r.request
+                req.headers['Authorization'] = "Negotiate %s" % response
+
+                # Right now there is no way to be sure to reply using the same socket connection
+                # since, after all, we are using a connection pool. So we try
+                # to release the conn and immediatly request a connection which will _likely_ 
+                # be the same as long as there are no concurrent requests using our session
+                # (well one way would be to implement a KerberosConnectionpool for urllib3, but
+                # then requests needs some patching as well to use that pool).
+
+                # shed unread content to be able to release connection back to the pool
+                r.content
+                r.raw.release_conn()
+                log.info("REQUESTS version %s", reqver)
+                if reqver.split(".")[0] == "0":
+                    r.request.send(anyway=True)
+                    r2 = r.request.response
+                else:
+                    r2 = r.connection.send(req
+                                           , verify = self.sslverify
+                                           , proxies = self.proxies
+                                           )
+                r2.history.append(r)
+                ret = self.handle_401(r2)
 
-                ret = _r
             if result == k.AUTH_GSS_COMPLETE and self.context:
-                 self.gss_clean(self.context)
-                 self.context = None
+                log.info("auth complete, now cleaning context...")
+                self.gss_clean(self.context)
+                self.context = None
 
         return ret
 
@@ -147,6 +175,7 @@ class HTTPKerberosAuth(AuthBase):
         return r
 
 def test(args):
+    from requests import session
     log.setLevel(logging.DEBUG)
     log.info("starting test")
     if args.keytab:
@@ -155,7 +184,8 @@ def test(args):
     r=s.get(args.url)
     print r.text
 #    if website is set up to keep auth, the next calls will not authenticate again
-#    (in IIS accomplish this by setting the windowsAuthentication properties: authPersistNonNTLM="true" and authPersistSingleRequest="false")
+#    (in IIS accomplish this by setting the windowsAuthentication properties: 
+#       authPersistNonNTLM="true" and authPersistSingleRequest="false")
 #    for i in range(20):
 #        r=s.get(args.url)
 #        print i, r.text
